/*
*
WIRELESS MIDI CONTROLLER CON INTERFACCIA ACCESSIBILE
*
*/

/*
LIBRERIE ESTERNE
*/

//Gestione LCD
#include <LiquidCrystal_I2C.h>

//Gestione comunicazione I2C
#include <Wire.h>

//Gestione comunicazione wireless
#include <WiFiUdp.h>
#include <WiFiServer.h>
#include <WiFiClient.h>
#include <WiFi.h>

//Gestione messaggi midi wireless/wired
#include <AppleMidi.h>
#include <MIDI.h>

//Gestione EEPROM microcontrollore
#include <EEPROM.h>

//Gestione men
#include <MenuBackend.h>

/*
*
VARIABILI
*
*/

//Variabili per la gestione della comunicazione wireless
//Modificare i valori di ssid e password per cambiare rete LAN
char ssid[50] = "ssid rete";
char password[50] = "password";
const char* host = "host.com";

//Host wireless non connesso
bool isConnected = false;

//Istanze di MIDI e AppleMIDI
MIDI_CREATE_INSTANCE(HardwareSerial, Serial, MIDI);
APPLEMIDI_CREATE_INSTANCE(WiFiUDP, AppleMIDI);

//Settaggio pin Arduinio mega
const int sensorPosition = A0;        // pin that's connected to position sensor
const int sensorForce = A1;           // pin that's connected to force sensor
const int buttonPinUp = 9;            // pin for the Up button
const int buttonPinDown = 10;         // pin for the Down button
const int buttonPinEsc = 11;          // pin for the Esc button
const int buttonPinEnter = 12;        // pin for the Enter button

//Valore di security range per il sensore di pressione
const int PressureSecurityRange = 5;

//Pin led di controllo 
const int ledOnOff = 2;
const int ledPlayng = 3;
const int ledSetting = 4;
const int ledMode1 = 5;
const int ledMode2 = 6;
const int ledWired = 7;
const int ledWifi = 8;

//Settaggio indirizzi memorizzazione EEPROM
const int addrSensorForceMin = 1;      //address in EEPROM to store Force Sensor minimum value
const int addrSensorForceMax = 2;      //address in EEPROM to store Force Sensor maximum value
const int addrSensorPosCC = 3;         //address in EEPROM to store Position Sensor CC
const int addrSensorForceCC = 4;       //address in EEPROM to store Force Sensor CC
const int addrMinNote = 5;             //address in EEPROM to store minimum note
const int addrMaxNote = 6;             //address in EEPROM to store maximum note
const int addrMidiChannel = 7;         //address in EEPROM to store midi channel
const int addrPlayMode = 8;            //address in EEPROM to store the play mode
const int addrSecurityRange = 10;      //address in EEPROM to store the value of security range
									   
//Costanti di identificazione degli stati del controller
const int statusPlayMode1 = 1;
const int statusPlayMode2 = 2;
const int statusSetCCPosition = 5;
const int statusSetCCForce = 6;
const int statusSetMinNote = 7;
const int statusSetMaxNote = 8;
const int statusMidiTeach = 9;
const int statusSetMidiChannel = 10;
const int statusSendWired = 11;
const int statusSendWireless = 12;
const int statusUseHand = 13;
const int statusUseFinger = 14;

int midiChannel;

//Stringhe per gestire le voci di men
char strMenu[21] = "        MENU        ";
char strMenuMidi[21] = "   MIDI SETUP MENU  ";
char strMenuModes[21] = "  MODES SETUP MENU  ";
char strSetSendMode[21] = "  SENDING MODE MENU ";
char strSettingWifi[21] = "   WIFI SETUP MENU  ";
char strSetMinimumNote[21] = "Set minimum note    ";
char strSetMaximumNote[21] = "Set maximum note    ";
char strSetPositionCC[21] = "Set position CC";
char strSetForceCC[21] = "Set force CC        ";
char strSetMidiChannel[21] = "Set MIDI channel    ";
char strSetUsingMode[21] = "Set Using Mode      ";

//Stringhe interfaccia principale modalit di comunicazione
char str11[6] = "Wired";
char str12[6] = "Wi-Fi";


//Settaggio valore LOW per i 4 pulsanti
int lastButtonPushed = 0;
int lastButtonEnterState = LOW;   // the previous reading from the Enter input pin
int lastButtonEscState = LOW;     // the previous reading from the input pin
int lastButtonUpState = LOW;      // the previous reading from the input pin
int lastButtonDownState = LOW;    // the previous reading from the input pin

//Variabili per calcolo del tempo trascorso dall'ultima pressione dei tasti (evita problemi di false pressioni)
long lastEnterDebounceTime = 0;  // the last time the output pin was toggled
long lastEscDebounceTime = 0;    // the last time the output pin was toggled
long lastUpDebounceTime = 0;     // the last time the output pin was toggled
long lastDownDebounceTime = 0;   // the last time the output pin was toggled

//Delay necassario affinch una pressione sia considerata tale
long debounceDelay = 500;    // the debounce time; increase if the output flickers

int currentStatusMode;
int currentUsingMode;
int currentSendingMode = 11; //default wired
int lastPlayingMode;

//Security range for finger and hand mode
int finger = 5;
int hand = 150;

//Midi variables
int ccPosition;
int ccForce;
int minNote = 80;
int maxNote = 100;
int noteVelocity = 100;
int ccPositionSetting;
int ccForceSetting;
int minNoteSetting;
int maxNoteSetting;
int midiChannelSetting;
int securityRange;
int id;    //0 per SSID e 1 per Password

int sensorForceValue = 0;             // the force sensor value
int sensorPositionValue = 0;          // the position sensor value
int sensorForceMinValue;              // minimum sensor value
int sensorForceMaxValue;              // maximum sensor value
int sensorPositionMinValue = 0;       // minimum sensor value
int sensorPositionMaxValue = 1023;    // maximum sensor value
int sensorPositionLastValue = 0;

//controller value for volume and pitch led 
int LedVolumeValue;
int OldLedVolumeValue;

int LedPitchValue;
int OldLedPitchValue;

int lastNote = 0;

int sensorForceRawValue = 0;           // the force sensor value
int sensorPositionRawValue = 0;        // the position sensor value

int sensorForcePreviousValue = -1;
int sensorPositionPreviousValue = -1;

//Inizializzazione LCD (indirizzo I2C 27, 20 colonne, 4 righe)
LiquidCrystal_I2C lcd(0x27, 20, 4);

static void menuUsed(MenuUseEvent used);
static void menuChanged(MenuChangeEvent changed);

MenuBackend menu = MenuBackend(menuUsed, menuChanged);

//Inizializzazione items di men
MenuItem menuModesSetup = MenuItem("menuModesSetup");
MenuItem menuItemMode1 = MenuItem("menuItemMode1");
MenuItem menuItemMode2 = MenuItem("menuItemMode2");
MenuItem menuUsingSetup = MenuItem("menuUsingSetup");
MenuItem menuItemUse1 = MenuItem("menuItemUse1");
MenuItem menuItemUse2 = MenuItem("menuItemUse2");
MenuItem menuItemMidiSetup = MenuItem("menuItemMidiSetup");
MenuItem menuItemCCPosition = MenuItem("menuItemCCPosition");
MenuItem menuItemCCForce = MenuItem("menuItemCCForce");
MenuItem menuItemMinNote = MenuItem("menuItemMinNote");
MenuItem menuItemMaxNote = MenuItem("menuItemMaxNote");
MenuItem menuItemMidiChannel = MenuItem("menuItemMidiChannel");
MenuItem menuItemMidiTeach = MenuItem("menuItemMidiTeach");
MenuItem menuSendMode = MenuItem("menuSendMode");
MenuItem menuItemSend1 = MenuItem("menuItemSend1");
MenuItem menuItemSend2 = MenuItem("menuItemSend2");
MenuItem menuItemWifiSetup = MenuItem("menuItemWifiSetup");
MenuItem menuItemWifi1 = MenuItem("menuItemWifi1");
MenuItem menuItemWifi2 = MenuItem("menuItemWifi2");
MenuItem menuItemWifi3 = MenuItem("menuItemWifi3");
MenuItem menuCalibration = MenuItem("menuCalibration");

/*
*
SETUP FUNCTION
*
*/

void setup()
{

	//Inizializzazione seriale a 9600 bps
	Serial.begin(9600);

	//Settaggio modalit pin 
	pinMode(sensorForce, INPUT);
	pinMode(sensorPosition, INPUT);
	pinMode(buttonPinUp, INPUT);
	pinMode(buttonPinDown, INPUT);
	pinMode(buttonPinEnter, INPUT);
	pinMode(buttonPinEsc, INPUT);

	//led controllo
	pinMode(ledOnOff, OUTPUT);
	pinMode(ledPlayng, OUTPUT);
	pinMode(ledSetting, OUTPUT);
	pinMode(ledMode1, OUTPUT);
	pinMode(ledMode2, OUTPUT);
	pinMode(ledWired, OUTPUT);
	pinMode(ledWifi, OUTPUT);

	//led volume
	pinMode(30, OUTPUT);
	pinMode(32, OUTPUT);
	pinMode(34, OUTPUT);
	pinMode(36, OUTPUT);
	pinMode(38, OUTPUT);
	pinMode(40, OUTPUT);
	pinMode(42, OUTPUT);
	pinMode(44, OUTPUT);
	pinMode(46, OUTPUT);
	pinMode(48, OUTPUT);
	pinMode(50, OUTPUT);
	pinMode(52, OUTPUT);

	//led pitch
	pinMode(31, OUTPUT);
	pinMode(33, OUTPUT);
	pinMode(35, OUTPUT);
	pinMode(37, OUTPUT);
	pinMode(39, OUTPUT);
	pinMode(41, OUTPUT);
	pinMode(43, OUTPUT);
	pinMode(45, OUTPUT);
	pinMode(47, OUTPUT);
	pinMode(49, OUTPUT);
	pinMode(51, OUTPUT);
	pinMode(53, OUTPUT);

	//Inizializzazione dei led a LOW per evitare false accensioni
	for (int i = 2; i < 9; i++) {
		digitalWrite(i, LOW);
	}

	for (int i = 30; i < 55; i++) {
		digitalWrite(i, LOW);
	}

	//accensione len Power
	digitalWrite(ledOnOff, HIGH);

	//Inizializzazione LCD
	lcd.init();
	lcd.backlight();  //retroilluminazione lcd


	int pressed = LOW;

	//All'accensione attendo 5 sec per permettere di resettare il controller
	while (millis()<5000 && pressed == LOW) {
		lcd.setCursor(0, 0);
		lcd.print("Push enter to");
		lcd.setCursor(0, 1);
		lcd.print("reset controller");
		pressed = digitalRead(buttonPinEnter);

	}

	sensorForceMinValue = EEPROM.read(addrSensorForceMin); //read from EEPROM the stored minimum force sensor value

	//if settings are not stored or Enter has been pressed during startup reset defaults and start calibration
	if (sensorForceMinValue == 255 || pressed == HIGH) {  
		calibration();

		//Dopo la calibrazione del sensore di pressione setto i valori di defoult delle altre variabili
		ccPosition = 20;                     //default
		ccForce = 21;                        //default
		minNote = 80;                        //default
		maxNote = 100;                       //default
		midiChannel = 1;                     //default
		securityRange = finger;              //default
		currentStatusMode = statusPlayMode1; //default
		currentSendingMode = statusSendWired;//default wired
		currentUsingMode = statusUseFinger;
		//scrivo i valori appena settati nelle rispettive celle EEPROM
		EEPROM.write(addrSensorPosCC, ccPosition);
		EEPROM.write(addrSensorForceCC, ccForce);
		EEPROM.write(addrMinNote, minNote);
		EEPROM.write(addrMaxNote, maxNote);
		EEPROM.write(addrMidiChannel, midiChannel);
		EEPROM.write(addrPlayMode, currentStatusMode);
		EEPROM.write(addrSecurityRange, finger);

		//Accendo i led opportuni
		digitalWrite(ledPlayng, HIGH);
		digitalWrite(ledSetting, LOW);
		digitalWrite(ledMode1, HIGH);
		digitalWrite(ledMode2, LOW);
		digitalWrite(ledWired, HIGH);
		digitalWrite(ledWifi, LOW);
	}
	else {

		//se non si effettua la calibrazione leggo i vecchi valori memorizzati
		sensorForceMinValue = sensorForceMinValue * 4;
		sensorForceMaxValue = (EEPROM.read(addrSensorForceMax)) * 4;
		ccPosition = EEPROM.read(addrSensorPosCC);      //read fromm EEPROM the stored force sensor CC value
		ccForce = EEPROM.read(addrSensorForceCC);       //read fromm EEPROM the stored force sensor CC value
		minNote = EEPROM.read(addrMinNote);             //read fromm EEPROM the stored minimum note
		maxNote = EEPROM.read(addrMaxNote);             //read fromm EEPROM the stored maximum note
		midiChannel = EEPROM.read(addrMidiChannel);     //read fromm EEPROM the stored midi channel
		currentSendingMode = statusSendWired;           //default wired
		currentStatusMode = EEPROM.read(addrPlayMode);  //read from EEPROM the stored play mode
		securityRange = EEPROM.read(addrSecurityRange); //read from EEPROM the stored security range

		digitalWrite(ledPlayng, HIGH);
		digitalWrite(ledSetting, LOW);

		if (currentStatusMode == 1) {
			digitalWrite(ledMode1, HIGH);
			digitalWrite(ledMode2, LOW);
		}
		else {
			digitalWrite(ledMode1, LOW);
			digitalWrite(ledMode2, HIGH);
		}

		if (securityRange == hand) {
			currentUsingMode = statusUseHand;
		}
		else {
			currentUsingMode = statusUseFinger;
		}

		digitalWrite(ledWired, HIGH);
		digitalWrite(ledWifi, LOW);

	}

	MIDI.begin(midiChannel); //initialize MIDI

    //menu configuration
	menu.getRoot().addRight(menuModesSetup).add(menuUsingSetup).add(menuItemMidiSetup).add(menuSendMode).add(menuItemWifiSetup).add(menuCalibration);
	menuModesSetup.addRight(menuItemMode1).add(menuItemMode2);
	menuUsingSetup.addRight(menuItemUse1).add(menuItemUse2);
	menuItemMidiSetup.addRight(menuItemCCPosition).add(menuItemCCForce).add(menuItemMinNote).add(menuItemMaxNote).add(menuItemMidiChannel).add(menuItemMidiTeach);
	menuSendMode.addRight(menuItemSend1).add(menuItemSend2);
	menuItemWifiSetup.addRight(menuItemWifi1).add(menuItemWifi2).add(menuItemWifi3);
	menu.toRoot();

}  // setup()...

/*
*
LOOP FUNCTION
*
*/

void loop()
{

	readButtons(); //read buttons status

	char strSetting[21] = "CC ";
	char sett[4];

	if (currentSendingMode == 12) {
		AppleMIDI.run();
	}

	//A seconda dello stato 
	switch (currentStatusMode) {
	
	/*
	*
	MODE 1
	*
	*/
	case statusPlayMode1: 

		navigateMenus();  //Navigate menus

		// sensors reading:
		sensorPositionRawValue = analogRead(sensorPosition);

		// if starts the touch. I added 10 to the minum just to avoid sensor little variations 
		if (sensorPositionRawValue>(sensorPositionMinValue + 10)) {

			delay(5); //this delay is really important to let the position sensor settle its resistance

			sensorPositionRawValue = analogRead(sensorPosition); //read the settled sensor position
			sensorPositionValue = map(sensorPositionRawValue, sensorPositionMinValue, sensorPositionMaxValue, minNote, maxNote);// apply the calibration to the position sensor reading
			sensorPositionValue = constrain(sensorPositionValue, minNote, maxNote); // in case the position sensor value is outside the range seen during calibration

			//Calcolo il valore del led associato al pitch della nota suonata
			LedPitchValue = map(sensorPositionRawValue, sensorPositionMinValue, sensorPositionMaxValue, 1, 12);

			//Ripeto la procedura per il sensore di pressione
			sensorForceRawValue = analogRead(sensorForce);
			sensorForceValue = map(sensorForceRawValue, sensorForceMinValue + 200, sensorForceMaxValue, 0, 127); // apply the calibration to force sensor reading
			sensorForceValue = constrain(sensorForceValue, 0, 127); // in case the position sensor value is outside the range seen during calibration

			//Calcolo il valore del led associato al volume in esecuzione
			LedVolumeValue = map(sensorForceRawValue, sensorForceMinValue, sensorForceMaxValue, 1, 12);

			//Invio dei messaggi MIDI a seconda del metodo di comunicazione
			if (currentSendingMode == 12 && isConnected == true) {
				AppleMIDI.noteOn(sensorPositionValue, noteVelocity, midiChannel);
				AppleMIDI.controlChange(ccForce, sensorForceValue, midiChannel);
			}
			else {
				MIDI.sendNoteOn(sensorPositionValue, noteVelocity, midiChannel);  // Send a Note (pitch based on position, fixed velocity)
				MIDI.sendControlChange(ccForce, sensorForceValue, midiChannel); // send midi control change defined by force sensor
			}

			//Accensione dei led relativi 
			for (int i = 31; i < (31 + (LedPitchValue * 2)); i = i + 2) {
				digitalWrite(i, HIGH);
			}

			for (int i = 30; i < (30 + (LedVolumeValue * 2)); i = i + 2) {
				digitalWrite(i, HIGH);
			}

			OldLedVolumeValue = LedVolumeValue;
			OldLedPitchValue = LedPitchValue;
			lastNote = sensorPositionValue;  // register the played note in order to stop it later

			lcd.setCursor(0, 3);
			lcd.print("   Sending Data...  ");

			do {
				sensorForceRawValue = analogRead(sensorForce);
				sensorPositionRawValue = analogRead(sensorPosition);
				sensorPositionLastValue = sensorPositionRawValue; //records the position sensor value before processing in order to use it later for the loop condition
				sensorForceValue = map(sensorForceRawValue, sensorForceMinValue, sensorForceMaxValue, 0, 127);  // apply the calibration to force sensor reading
				sensorForceValue = constrain(sensorForceValue, 0, 127); // in case the position sensor value is outside the range seen during calibration

				LedVolumeValue = map(sensorForceRawValue, sensorForceMinValue, sensorForceMaxValue, 1, 12);

				if (sensorPositionRawValue>(sensorPositionMinValue)) {

					LedPitchValue = map(sensorPositionRawValue, sensorPositionMinValue, sensorPositionMaxValue, 1, 12);

					if ((sensorPositionRawValue<(sensorPositionPreviousValue - securityRange)) || (sensorPositionRawValue>(sensorPositionPreviousValue + securityRange))) { //ignores little variations

						sensorPositionPreviousValue = sensorPositionRawValue;  //records input value to check little variations at next do
						sensorPositionValue = map(sensorPositionRawValue, sensorPositionMinValue, sensorPositionMaxValue, minNote, maxNote);// apply the calibration to the position sensor reading
						sensorPositionValue = constrain(sensorPositionValue, minNote, maxNote);// in case the position sensor value is outside the range seen during calibration  

						if (sensorPositionValue != lastNote) { //if changed 
							if (currentSendingMode == 12 && isConnected == true) {
								AppleMIDI.noteOff(lastNote, 0, midiChannel);
								AppleMIDI.noteOn(sensorPositionValue, 100, midiChannel);
							}
							else {
								MIDI.sendNoteOff(lastNote, 0, midiChannel);   // Stop the note
								MIDI.sendNoteOn(sensorPositionValue, 100, midiChannel);  // Send a Note (pitch 42, velo 127 on channel 1)
							}

							if (LedPitchValue >= OldLedPitchValue) {
								for (int i = 31; i < (31 + (LedPitchValue * 2)); i = i + 2) {
									digitalWrite(i, HIGH);
								}
							}
							else {
								int diff = OldLedPitchValue - LedPitchValue;
								for (int i = (29 + (OldLedPitchValue * 2)); i >(29 + (LedPitchValue * 2)); i = i - 2) {
									digitalWrite(i, LOW);
								}
							}

							OldLedPitchValue = LedPitchValue;
							lastNote = sensorPositionValue; // register the played note in order to stop it later   
						}
					}

					if ((sensorForceValue != sensorForcePreviousValue) && (sensorForceRawValue>(sensorForceMinValue + PressureSecurityRange))) {
						sensorForcePreviousValue = sensorForceValue;
						if (currentSendingMode == 12 && isConnected == true) {
							AppleMIDI.controlChange(ccForce, sensorForceValue, midiChannel);
						}
						else {
							MIDI.sendControlChange(ccForce, sensorForceValue, midiChannel);// send midi control change defined by force sensor
						}

						if (LedVolumeValue >= OldLedVolumeValue) {
							for (int i = 30; i < (30 + (LedVolumeValue * 2)); i = i + 2) {
								digitalWrite(i, HIGH);
							}
						}
						else {
							int diff = OldLedVolumeValue - LedVolumeValue;
							for (int i = (28 + (OldLedVolumeValue * 2)); i >(28 + (LedVolumeValue * 2)); i = i - 2) {
								digitalWrite(i, LOW);
							}
						}

						OldLedVolumeValue = LedVolumeValue;
					}
				}
			} while (sensorPositionLastValue>(sensorPositionMinValue + 10)); //loops till the sensor is pressed

			if (currentSendingMode == 12 && isConnected == true) {
				AppleMIDI.noteOff(lastNote, 0, midiChannel);
				AppleMIDI.controlChange(ccForce, 0, midiChannel);
			}
			else {
				MIDI.sendNoteOff(lastNote, 0, midiChannel);   // Stop the note
				MIDI.sendControlChange(ccForce, 0, midiChannel);
			}

			for (int i = (28 + (OldLedVolumeValue * 2)); i > 28; i = i - 2) {
				digitalWrite(i, LOW);
			}
			for (int i = (29 + (OldLedPitchValue * 2)); i > 29; i = i - 2) {
				digitalWrite(i, LOW);
			}

			lcd.setCursor(0, 3);
			lcd.print("   Waiting Data...  ");
		}
		break;

	/*
	*
	MODE 2
	*
	*/

	case statusPlayMode2:   //playing mode 2

		navigateMenus();//Navigate menus

		sensorPositionRawValue = analogRead(sensorPosition);// position sensor reading:

		if (sensorPositionRawValue>(sensorPositionMinValue + 10)) { // if starts the touch. I added 20 to hte minum just to avoid sensor little variations 

			delay(5); //this delay is really important to let the position sensor settle its resistance

			sensorPositionRawValue = analogRead(sensorPosition); //read the settled sensor position
			sensorPositionValue = map(sensorPositionRawValue, sensorPositionMinValue, sensorPositionMaxValue, minNote, maxNote);// apply the calibration to the position sensor reading and calculates the corrisponding value for midi command
			sensorPositionValue = constrain(sensorPositionValue, minNote, maxNote);// in case the position sensor value is outside the range seen during calibration

			LedPitchValue = map(sensorPositionRawValue, sensorPositionMinValue, sensorPositionMaxValue, 1, 12);

			if (currentSendingMode == 12 && isConnected == true) {
				AppleMIDI.noteOn(sensorPositionValue, noteVelocity, midiChannel);
			}
			else {
				MIDI.sendNoteOn(sensorPositionValue, noteVelocity, midiChannel);  // Send a Note (pitch based on position, fixed velocity)
			}

			for (int i = 31; i < (31 + (LedPitchValue * 2)); i = i + 2) {
				digitalWrite(i, HIGH);
			}

			OldLedPitchValue = LedPitchValue;
			OldLedVolumeValue = 0;
			lastNote = sensorPositionValue;// register the played note in order to stop it later

			lcd.setCursor(0, 3);
			lcd.print("   Sending Data...  ");

			do {
				sensorForceRawValue = analogRead(sensorForce);
				sensorPositionRawValue = analogRead(sensorPosition);
				sensorPositionLastValue = sensorPositionRawValue;//registers the position sensor value before processing in order to use it later for the loop condition
				sensorForceValue = map(sensorForceRawValue, sensorForceMinValue, sensorForceMaxValue, 0, 127);// apply the calibration to force sensor reading
				sensorForceValue = constrain(sensorForceValue, 0, 127);// in case the position sensor value is outside the range seen during calibration

				LedVolumeValue = map(sensorForceRawValue, sensorForceMinValue, sensorForceMaxValue, 1, 12);

				if (sensorPositionRawValue>(sensorPositionMinValue)) {
					sensorPositionValue = map(sensorPositionRawValue, sensorPositionMinValue, sensorPositionMaxValue, minNote, maxNote);// apply the calibration to the position sensor reading
					sensorPositionValue = constrain(sensorPositionValue, minNote, maxNote); // in case the position sensor value is outside the range seen during calibration

					LedPitchValue = map(sensorPositionRawValue, sensorPositionMinValue, sensorPositionMaxValue, 1, 12);

					if (currentSendingMode == 12 && isConnected == true) {
						AppleMIDI.controlChange(ccPosition, sensorPositionValue, midiChannel);
						AppleMIDI.controlChange(ccForce, sensorForceValue, midiChannel);
					}
					else {
						MIDI.sendControlChange(ccPosition, sensorPositionValue, midiChannel);// send midi control change defined by position sensor
						MIDI.sendControlChange(ccForce, sensorForceValue, midiChannel);// send midi control change defined by force sensor
					}

					if (LedVolumeValue >= OldLedVolumeValue) {
						for (int i = 30; i < (30 + (LedVolumeValue * 2)); i = i + 2) {
							digitalWrite(i, HIGH);
						}
					}
					else {
						int diff = OldLedVolumeValue - LedVolumeValue;
						for (int i = (28 + (OldLedVolumeValue * 2)); i >(28 + (LedVolumeValue * 2)); i = i - 2) {
							digitalWrite(i, LOW);
						}
					}

					if (LedPitchValue >= OldLedPitchValue) {
						for (int i = 31; i < (31 + (LedPitchValue * 2)); i = i + 2) {
							digitalWrite(i, HIGH);
						}
					}
					else {
						int diff = OldLedPitchValue - LedPitchValue;
						for (int i = (29 + (OldLedPitchValue * 2)); i >(29 + (LedPitchValue * 2)); i = i - 2) {
							digitalWrite(i, LOW);
						}
					}

					OldLedPitchValue = LedPitchValue;
					OldLedVolumeValue = LedVolumeValue;
				}
			} while (sensorPositionLastValue>(sensorPositionMinValue + 10)); //loops till the sensor is pressed

			if (currentSendingMode == 12 && isConnected == true) {
				AppleMIDI.noteOff(lastNote, 0, midiChannel);
				AppleMIDI.controlChange(ccForce, 0, midiChannel);
			}
			else {
				MIDI.sendNoteOff(lastNote, 0, midiChannel);   // Stop the note
				MIDI.sendControlChange(ccForce, 0, midiChannel);
			}

			for (int i = (28 + (OldLedVolumeValue * 2)); i > 28; i = i - 2) {
				digitalWrite(i, LOW);
			}

			for (int i = (29 + (OldLedPitchValue * 2)); i > 29; i = i - 2) {
				digitalWrite(i, LOW);
			}

			lcd.setCursor(0, 3);
			lcd.print("   Waiting Data...  ");
		}
		break;

	/*
	*
	SETTAGGIO CONTROL CHANGE POSIZIONE
	*
	*/

	case statusSetCCPosition:    //setting position cc  
		switch (lastButtonPushed) {
		case buttonPinEnter: //set the new position cc and go back to the main screen
			ccPosition = ccPositionSetting;
			currentStatusMode = lastPlayingMode;
			EEPROM.write(addrSensorPosCC, ccPosition);
			digitalWrite(ledPlayng, HIGH);
			digitalWrite(ledSetting, LOW);
			menu.toRoot();
			break;
		case buttonPinEsc: //go back to the main screen
			ccPositionSetting = ccPosition;
			currentStatusMode = lastPlayingMode;
			digitalWrite(ledPlayng, HIGH);
			digitalWrite(ledSetting, LOW);
			menu.toRoot();
			break;
		case buttonPinUp: //increase position cc
			if (ccPositionSetting<127) {
				ccPositionSetting = ccPositionSetting + 1;
				itoa(ccPositionSetting, sett, 10); //conversion needed to concatenate the value to a string in order to print it on lcd
				lcd.clear();
				lcd.setCursor(0, 0);
				lcd.print(strSetPositionCC);
				lcd.setCursor(0, 1);
				lcd.print(strcat(strSetting, sett));
			}
			break;
		case buttonPinDown://decrease position cc
			if (ccPositionSetting>0) {
				ccPositionSetting = ccPositionSetting - 1;
				itoa(ccPositionSetting, sett, 10);//conversion needed to concatenate the value to a string in order to print it on lcd
				lcd.clear();
				lcd.setCursor(0, 0);
				lcd.print(strSetPositionCC);
				lcd.setCursor(0, 1);
				lcd.print(strcat(strSetting, sett));
			}
			break;
		}
		break;

	/*
	*
	SETTAGGIO CONTROL CHANGE PRESSIONE
	*
	*/

	case statusSetCCForce:    //set force cc
		switch (lastButtonPushed) {
		case buttonPinEnter://set the new force cc and go back to the main screen
			ccForce = ccForceSetting;
			currentStatusMode = lastPlayingMode;
			EEPROM.write(addrSensorForceCC, ccForce);
			digitalWrite(ledPlayng, HIGH);
			digitalWrite(ledSetting, LOW);
			menu.toRoot();
			break;
		case buttonPinEsc://go back to the main screen
			ccForceSetting = ccForce;
			currentStatusMode = lastPlayingMode;
			digitalWrite(ledPlayng, HIGH);
			digitalWrite(ledSetting, LOW);
			menu.toRoot();
			break;
		case buttonPinUp://increase force cc
			if (ccForceSetting<127) {
				ccForceSetting = ccForceSetting + 1;
				itoa(ccForceSetting, sett, 10);   //conversion needed to concatenate the value to a string in order to print it on lcd 
				lcd.clear();
				lcd.setCursor(0, 0);
				lcd.print(strSetForceCC);
				lcd.setCursor(0, 1);
				lcd.print(strcat(strSetting, sett));
			}
			break;
		case buttonPinDown://decrease force cc
			if (ccForceSetting>0) {
				ccForceSetting = ccForceSetting - 1;
				itoa(ccForceSetting, sett, 10);//conversion needed to concatenate the value to a string in order to print it on lcd
				lcd.clear();
				lcd.setCursor(0, 0);
				lcd.print(strSetForceCC);
				lcd.setCursor(0, 1);
				lcd.print(strcat(strSetting, sett));
			}
			break;
		}
		break;

	/*
	*
	SETTAGGIO NOTA MINIMA
	*
	*/
	case statusSetMinNote:    //set min note 
		switch (lastButtonPushed) {
		case buttonPinEnter://set the new minimum note and go back to the main screen
			minNote = minNoteSetting;
			currentStatusMode = lastPlayingMode;
			EEPROM.write(addrMinNote, minNote);
			digitalWrite(ledPlayng, HIGH);
			digitalWrite(ledSetting, LOW);
			menu.toRoot();
			break;
		case buttonPinEsc://go back to the main screen
			minNoteSetting = minNote;
			currentStatusMode = lastPlayingMode;
			digitalWrite(ledPlayng, HIGH);
			digitalWrite(ledSetting, LOW);
			menu.toRoot();
			break;
		case buttonPinUp://increase minimum note
			if (minNoteSetting<127) {
				minNoteSetting = minNoteSetting + 1;
				lcd.clear();
				lcd.setCursor(0, 0);
				lcd.print(strSetMinimumNote);
				lcd.setCursor(0, 1);
				lcd.print(minNoteSetting);
			}
			break;
		case buttonPinDown://decrease minimum note
			if (minNoteSetting>0) {
				minNoteSetting = minNoteSetting - 1;
				lcd.clear();
				lcd.setCursor(0, 0);
				lcd.print(strSetMinimumNote);
				lcd.setCursor(0, 1);
				lcd.print(minNoteSetting);
			}
			break;
		}
		break;

	/*
	*
	SETTAGGIO NOTA MASSIMA
	*
	*/

	case statusSetMaxNote:    //set max note
		switch (lastButtonPushed) {
		case buttonPinEnter://set the new maximum note and go back to the main screen
			maxNote = maxNoteSetting;
			currentStatusMode = lastPlayingMode;
			EEPROM.write(addrMaxNote, maxNote);
			digitalWrite(ledPlayng, HIGH);
			digitalWrite(ledSetting, LOW);
			menu.toRoot();
			break;
		case buttonPinEsc://go back to the main screen
			maxNoteSetting = maxNote;
			currentStatusMode = lastPlayingMode;
			digitalWrite(ledPlayng, HIGH);
			digitalWrite(ledSetting, LOW);
			menu.toRoot();
			break;
		case buttonPinUp://increase maximum note
			if (maxNoteSetting<127) {
				maxNoteSetting = maxNoteSetting + 1;
				lcd.clear();
				lcd.setCursor(0, 0);
				lcd.print(strSetMaximumNote);
				lcd.setCursor(0, 1);
				lcd.print(maxNoteSetting);
			}
			break;
		case buttonPinDown://decrease maximum note
			if (maxNoteSetting>0) {
				maxNoteSetting = maxNoteSetting - 1;
				lcd.clear();
				lcd.setCursor(0, 0);
				lcd.print(strSetMaximumNote);
				lcd.setCursor(0, 1);
				lcd.print(maxNoteSetting);
			}
			break;
		}
		break;

	/*
	*
	SETTAGGIO MIDI CHANNEL
	*
	*/

	case statusSetMidiChannel:    //set midi channel
		switch (lastButtonPushed) {
		case buttonPinEnter://set the new midi channel and go back to the main screen
			midiChannel = midiChannelSetting;
			currentStatusMode = lastPlayingMode;
			EEPROM.write(addrMidiChannel, midiChannel);
			digitalWrite(ledPlayng, HIGH);
			digitalWrite(ledSetting, LOW);
			menu.toRoot();
			break;
		case buttonPinEsc://go back to the main screen
			midiChannelSetting = midiChannel;
			currentStatusMode = lastPlayingMode;
			digitalWrite(ledPlayng, HIGH);
			digitalWrite(ledSetting, LOW);
			menu.toRoot();
			break;
		case buttonPinUp://increase maximum note
			if (midiChannelSetting<16) {
				midiChannelSetting = midiChannelSetting + 1;
				lcd.clear();
				lcd.setCursor(0, 0);
				lcd.print(strSetMidiChannel);
				lcd.setCursor(0, 1);
				lcd.print(midiChannelSetting);
			}
			break;
		case buttonPinDown://decrease maximum note
			if (midiChannelSetting>1) {
				midiChannelSetting = midiChannelSetting - 1;
				lcd.clear();
				lcd.setCursor(0, 0);
				lcd.print(strSetMidiChannel);
				lcd.setCursor(0, 1);
				lcd.print(midiChannelSetting);
			}
			break;
		}
		break;

	/*
	*
	MIDI TEACH
	*
	*/

	case statusMidiTeach:    //midi teach
		switch (lastButtonPushed) {
		case buttonPinEsc://go back to the main screen
			currentStatusMode = lastPlayingMode;
			digitalWrite(ledPlayng, HIGH);
			digitalWrite(ledSetting, LOW);
			menu.toRoot();
			break;
		case buttonPinDown://send two control change command for force cc
			MIDI.sendControlChange(ccForce, 64, 1);
			MIDI.sendControlChange(ccForce, 65, 1);
			break;
		case buttonPinUp://send two control change command for position cc
			MIDI.sendControlChange(ccPosition, 64, 1);
			MIDI.sendControlChange(ccPosition, 65, 1);
			break;
		}
		break;
	}// switch currentStatusMode...
} //loop()... 

  /*
  *
  MENU
  *
  */

void menuChanged(MenuChangeEvent changed) {  //change text on lcd display while navigating
	MenuItem newMenuItem = changed.to;
	lcd.setCursor(0, 0);

	if (newMenuItem.getName() == menu.getRoot()) {
		char modeNumber[2];
		String invio;
		String uso;
		itoa(currentStatusMode, modeNumber, 10);//conversion needed to concatenate the value to a string in order to print it on lcd

		if (currentUsingMode == 13) {
			uso = "USING MODE: HAND";
		}
		else {
			uso = "USING MODE: FINGER";
		}

		if (currentSendingMode == 11) {
			invio = "SENDING MODE: WIRED";
		}
		else {
			invio = "SENDING MODE: Wi-Fi";
		}

		char strPlaying[17] = "PLAYING MODE: ";
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print(strcat(strPlaying, modeNumber));
		lcd.setCursor(0, 1);
		lcd.print(uso);
		lcd.setCursor(0, 2);
		lcd.print(invio);
		digitalWrite(ledPlayng, HIGH);
		digitalWrite(ledSetting, LOW);
	}
	else if (newMenuItem.getName() == "menuModesSetup") {
		lcd.print(strMenu);
		lcd.setCursor(0, 1);
		lcd.print("Modes Setup      <--");
		lcd.setCursor(0, 2);
		lcd.print("Using Setup         ");
		lcd.setCursor(0, 3);
		lcd.print("MIDI Setup          ");
		digitalWrite(ledPlayng, LOW);
		digitalWrite(ledSetting, HIGH);
	}
	else if (newMenuItem.getName() == "menuUsingSetup") {
		lcd.print(strMenu);
		lcd.setCursor(0, 1);
		lcd.print("Modes Setup         ");
		lcd.setCursor(0, 2);
		lcd.print("Using Setup      <--");
		lcd.setCursor(0, 3);
		lcd.print("MIDI Setup          ");
	}
	else if (newMenuItem.getName() == "menuItemMidiSetup") {
		lcd.print(strMenu);
		lcd.setCursor(0, 1);
		lcd.print("Modes Setup         ");
		lcd.setCursor(0, 2);
		lcd.print("Using Setup         ");
		lcd.setCursor(0, 3);
		lcd.print("MIDI Setup       <--");
	}
	else if (newMenuItem.getName() == "menuSendMode") {
		lcd.print(strMenu);
		lcd.setCursor(0, 1);
		lcd.print("Using Setup         ");
		lcd.setCursor(0, 2);
		lcd.print("MIDI Setup          ");
		lcd.setCursor(0, 3);
		lcd.print("Sending mode     <--");
	}
	else if (newMenuItem.getName() == "menuItemWifiSetup") {
		lcd.print(strMenu);
		lcd.setCursor(0, 1);
		lcd.print("MIDI Setup          ");
		lcd.setCursor(0, 2);
		lcd.print("Sending mode        ");
		lcd.setCursor(0, 3);
		lcd.print("Wi-Fi Setup      <--");
	}
	else if (newMenuItem.getName() == "menuCalibration") {
		lcd.print(strMenu);
		lcd.setCursor(0, 1);
		lcd.print("Sending mode        ");
		lcd.setCursor(0, 2);
		lcd.print("Wi-Fi Setup         ");
		lcd.setCursor(0, 3);
		lcd.print("Calibration      <--");
	}
	else if (newMenuItem.getName() == "menuItemMode1") {
		lcd.print(strMenuModes);
		lcd.setCursor(0, 1);
		lcd.print("                    ");
		lcd.setCursor(0, 2);
		lcd.print("Mode 1           <--");
		lcd.setCursor(0, 3);
		lcd.print("Mode 2              ");
	}
	else if (newMenuItem.getName() == "menuItemMode2") {
		lcd.print(strMenuModes);
		lcd.setCursor(0, 1);
		lcd.print("                    ");
		lcd.setCursor(0, 2);
		lcd.print("Mode 1              ");
		lcd.setCursor(0, 3);
		lcd.print("Mode 2           <--");
	}
	else if (newMenuItem.getName() == "menuItemUse1") {
		lcd.print(strSetUsingMode);
		lcd.setCursor(0, 1);
		lcd.print("                    ");
		lcd.setCursor(0, 2);
		lcd.print("Hand mode        <--");
		lcd.setCursor(0, 3);
		lcd.print("Finger mode         ");
	}
	else if (newMenuItem.getName() == "menuItemUse2") {
		lcd.print(strSetUsingMode);
		lcd.setCursor(0, 1);
		lcd.print("                    ");
		lcd.setCursor(0, 2);
		lcd.print("Hand mode           ");
		lcd.setCursor(0, 3);
		lcd.print("Finger mode      <--");
	}
	else if (newMenuItem.getName() == "menuItemCCPosition") {
		lcd.print(strMenuMidi);
		lcd.setCursor(0, 1);
		lcd.print("                    ");
		lcd.setCursor(0, 2);
		lcd.print("Set position CC  <--");
		lcd.setCursor(0, 3);
		lcd.print(strSetForceCC);
	}
	else if (newMenuItem.getName() == "menuItemCCForce") {
		lcd.print(strMenuMidi);
		lcd.setCursor(0, 1);
		lcd.print(strSetPositionCC);
		lcd.setCursor(0, 2);
		lcd.print("Set force CC     <--");
		lcd.setCursor(0, 3);
		lcd.print(strSetMinimumNote);
	}
	else if (newMenuItem.getName() == "menuItemMinNote") {
		lcd.print(strMenuMidi);
		lcd.setCursor(0, 1);
		lcd.print(strSetForceCC);
		lcd.setCursor(0, 2);
		lcd.print("Set minimum note <--");
		lcd.setCursor(0, 3);
		lcd.print(strSetMaximumNote);
	}
	else if (newMenuItem.getName() == "menuItemMaxNote") {
		lcd.print(strMenuMidi);
		lcd.setCursor(0, 1);
		lcd.print(strSetMinimumNote);
		lcd.setCursor(0, 2);
		lcd.print("Set maximum note <--");
		lcd.setCursor(0, 3);
		lcd.print(strSetMidiChannel);
	}
	else if (newMenuItem.getName() == "menuItemMidiChannel") {
		lcd.print(strMenuMidi);
		lcd.setCursor(0, 1);
		lcd.print(strSetMaximumNote);
		lcd.setCursor(0, 2);
		lcd.print("Set MIDI channel <--");
		lcd.setCursor(0, 3);
		lcd.print("MIDI teach          ");
	}
	else if (newMenuItem.getName() == "menuItemMidiTeach") {
		lcd.print(strMenuMidi);
		lcd.setCursor(0, 1);
		lcd.print(strSetMidiChannel);
		lcd.setCursor(0, 2);
		lcd.print("MIDI teach       <--");
		lcd.setCursor(0, 3);
		lcd.print("                    ");
	}
	else if (newMenuItem.getName() == "menuItemSend1") {
		lcd.print(strSetSendMode);
		lcd.setCursor(0, 1);
		lcd.print("                    ");
		lcd.setCursor(0, 2);
		lcd.print("Wired mode       <--");
		lcd.setCursor(0, 3);
		lcd.print("Wireless mode       ");
	}
	else if (newMenuItem.getName() == "menuItemSend2") {
		lcd.print(strSetSendMode);
		lcd.setCursor(0, 1);
		lcd.print("                    ");
		lcd.setCursor(0, 2);
		lcd.print("Wired mode          ");
		lcd.setCursor(0, 3);
		lcd.print("Wireless mode    <--");
	}
	else if (newMenuItem.getName() == "menuItemWifi1") {
		lcd.print(strSettingWifi);
		lcd.setCursor(0, 1);
		lcd.print("Set SSID         <--");
		lcd.setCursor(0, 2);
		lcd.print("Set Password        ");
		lcd.setCursor(0, 3);
		lcd.print("Set SSID & Password ");
	}
	else if (newMenuItem.getName() == "menuItemWifi2") {
		lcd.print(strSettingWifi);
		lcd.setCursor(0, 1);
		lcd.print("Set SSID            ");
		lcd.setCursor(0, 2);
		lcd.print("Set Password     <--");
		lcd.setCursor(0, 3);
		lcd.print("Set SSID & Password ");
	}
	else if (newMenuItem.getName() == "menuItemWifi3") {
		lcd.print(strSettingWifi);
		lcd.setCursor(0, 1);
		lcd.print("Set SSID            ");
		lcd.setCursor(0, 2);
		lcd.print("Set Password        ");
		lcd.setCursor(0, 3);
		lcd.print("Set SSID & Pass  <--");
	}
}

void menuUsed(MenuUseEvent used) {  //change status based on menu item used, or make some action
	if (used.item == menuItemMode1) {
		currentStatusMode = statusPlayMode1;
		EEPROM.write(addrPlayMode, statusPlayMode1);
		digitalWrite(ledMode1, HIGH);
		digitalWrite(ledMode2, LOW);
		digitalWrite(ledPlayng, HIGH);
		digitalWrite(ledSetting, LOW);
		menu.toRoot();
	}
	else if (used.item == menuItemMode2) {
		currentStatusMode = statusPlayMode2;
		EEPROM.write(addrPlayMode, statusPlayMode2);
		digitalWrite(ledMode1, LOW);
		digitalWrite(ledMode2, HIGH);
		digitalWrite(ledPlayng, HIGH);
		digitalWrite(ledSetting, LOW);
		menu.toRoot();
	}
	else if (used.item == menuItemUse1) {
		currentUsingMode = statusUseHand;
		securityRange = hand;
		EEPROM.write(addrSecurityRange, hand);
		menu.toRoot();
	}
	else if (used.item == menuItemUse2) {
		currentUsingMode = statusUseFinger;
		securityRange = finger;
		EEPROM.write(addrSecurityRange, finger);
		menu.toRoot();
	}
	else if (used.item == menuItemSend1) {
		currentSendingMode = statusSendWired;
		digitalWrite(ledWired, HIGH);
		digitalWrite(ledWifi, LOW);
		digitalWrite(ledPlayng, HIGH);
		digitalWrite(ledSetting, LOW);
		menu.toRoot();
	}
	else if (used.item == menuItemSend2) {
		currentSendingMode = statusSendWireless;
		digitalWrite(ledPlayng, LOW);
		digitalWrite(ledSetting, HIGH);
		SetWireless();
		digitalWrite(ledPlayng, HIGH);
		digitalWrite(ledSetting, LOW);
		menu.toRoot();
	}
	else if (used.item == menuItemWifi1) {
		digitalWrite(ledPlayng, LOW);
		digitalWrite(ledSetting, HIGH);
		SetWifiCredential(0);
		digitalWrite(ledPlayng, HIGH);
		digitalWrite(ledSetting, LOW);
		menu.toRoot();
	}
	else if (used.item == menuItemWifi2) {
		digitalWrite(ledPlayng, LOW);
		digitalWrite(ledSetting, HIGH);
		SetWifiCredential(1);
		digitalWrite(ledPlayng, HIGH);
		digitalWrite(ledSetting, LOW);
		menu.toRoot();
	}
	else if (used.item == menuItemWifi3) {
		digitalWrite(ledPlayng, LOW);
		digitalWrite(ledSetting, HIGH);
		SetWifiCredential(2);
		digitalWrite(ledPlayng, HIGH);
		digitalWrite(ledSetting, LOW);
		menu.toRoot();
	}
	else if (used.item == menuItemCCPosition) {
		ccPositionSetting = ccPosition;
		lastPlayingMode = currentStatusMode;
		currentStatusMode = statusSetCCPosition;
		digitalWrite(ledPlayng, LOW);
		digitalWrite(ledSetting, HIGH);
		char ccPos[4];
		itoa(ccPositionSetting, ccPos, 10);//conversion needed to concatenate the value to a string in order to print it on lcd
		char strCcPos[17] = "CC ";
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print(strSetPositionCC);
		lcd.setCursor(0, 1);
		lcd.print(strcat(strCcPos, ccPos));
	}
	else if (used.item == menuItemCCForce) {
		ccForceSetting = ccForce;
		lastPlayingMode = currentStatusMode;
		currentStatusMode = statusSetCCForce;
		digitalWrite(ledPlayng, LOW);
		digitalWrite(ledSetting, HIGH);
		char ccPress[4];
		itoa(ccForceSetting, ccPress, 10);//conversion needed to concatenate the value to a string in order to print it on lcd
		char strCcPress[17] = "CC ";
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print(strSetForceCC);
		lcd.setCursor(0, 1);
		lcd.print(strcat(strCcPress, ccPress));
	}
	else if (used.item == menuItemMinNote) {
		minNoteSetting = minNote;
		lastPlayingMode = currentStatusMode;
		currentStatusMode = statusSetMinNote;
		digitalWrite(ledPlayng, LOW);
		digitalWrite(ledSetting, HIGH);
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print(strSetMinimumNote);
		lcd.setCursor(0, 1);
		lcd.print(minNote);
	}
	else if (used.item == menuItemMaxNote) {
		maxNoteSetting = maxNote;
		lastPlayingMode = currentStatusMode;
		currentStatusMode = statusSetMaxNote;
		digitalWrite(ledPlayng, LOW);
		digitalWrite(ledSetting, HIGH);
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print(strSetMaximumNote);
		lcd.setCursor(0, 1);
		lcd.print(maxNote);
	}
	else if (used.item == menuItemMidiChannel) {
		midiChannelSetting = midiChannel;
		lastPlayingMode = currentStatusMode;
		currentStatusMode = statusSetMidiChannel;
		digitalWrite(ledPlayng, LOW);
		digitalWrite(ledSetting, HIGH);
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print(strSetMidiChannel);
		lcd.setCursor(0, 1);
		lcd.print(midiChannelSetting);
	}
	else if (used.item == menuItemMidiTeach) {
		lastPlayingMode = currentStatusMode;
		currentStatusMode = statusMidiTeach;
		digitalWrite(ledPlayng, LOW);
		digitalWrite(ledSetting, HIGH);
		lcd.setCursor(0, 0);
		lcd.print("MIDI TEACH MODE     ");
		lcd.setCursor(0, 1);
		lcd.print("Up: send position CC");
		lcd.setCursor(0, 2);
		lcd.print("Down: send force CC ");
	}
	else if (used.item == menuCalibration) {
		digitalWrite(ledPlayng, LOW);
		digitalWrite(ledSetting, HIGH);
		calibration();
		digitalWrite(ledPlayng, HIGH);
		digitalWrite(ledSetting, LOW);
		menu.toRoot();
	}
}

/*
*
LETTURA STATO BOTTONI
*
*/

void  readButtons() {  //read buttons status
	int reading;
	int buttonEnterState = LOW;            // the current reading from the Enter input pin
	int buttonEscState = LOW;              // the current reading from the input pin
	int buttonUpState = LOW;               // the current reading from the input pin
	int buttonDownState = LOW;             // the current reading from the input pin

	//Enter button
	// read the state of the switch into a local variable:
	reading = digitalRead(buttonPinEnter);

	// check to see if you just pressed the enter button 
	// (i.e. the input went from LOW to HIGH),  and you've waited 
	// long enough since the last press to ignore any noise:  

	// If the switch changed, due to noise or pressing:
	if (reading != lastButtonEnterState) {
		// reset the debouncing timer
		lastEnterDebounceTime = millis();
	}

	if ((millis() - lastEnterDebounceTime) > debounceDelay) {
		// whatever the reading is at, it's been there for longer
		// than the debounce delay, so take it as the actual current state:
		buttonEnterState = reading;
		lastEnterDebounceTime = millis();
	}

	// save the reading.  Next time through the loop,
	// it'll be the lastButtonState:
	lastButtonEnterState = reading;


	//Esc button               
	// read the state of the switch into a local variable:
	reading = digitalRead(buttonPinEsc);

	// check to see if you just pressed the Down button 
	// (i.e. the input went from LOW to HIGH),  and you've waited 
	// long enough since the last press to ignore any noise:  

	// If the switch changed, due to noise or pressing:
	if (reading != lastButtonEscState) {
		// reset the debouncing timer
		lastEscDebounceTime = millis();
	}

	if ((millis() - lastEscDebounceTime) > debounceDelay) {
		// whatever the reading is at, it's been there for longer
		// than the debounce delay, so take it as the actual current state:
		buttonEscState = reading;
		lastEscDebounceTime = millis();
	}

	// save the reading.  Next time through the loop,
	// it'll be the lastButtonState:
	lastButtonEscState = reading;


	//Down button               
	// read the state of the switch into a local variable:
	reading = digitalRead(buttonPinDown);

	// check to see if you just pressed the Down button 
	// (i.e. the input went from LOW to HIGH),  and you've waited 
	// long enough since the last press to ignore any noise:  

	// If the switch changed, due to noise or pressing:
	if (reading != lastButtonDownState) {
		// reset the debouncing timer
		lastDownDebounceTime = millis();
	}

	if ((millis() - lastDownDebounceTime) > debounceDelay) {
		// whatever the reading is at, it's been there for longer
		// than the debounce delay, so take it as the actual current state:
		buttonDownState = reading;
		lastDownDebounceTime = millis();
	}

	// save the reading.  Next time through the loop,
	// it'll be the lastButtonState:
	lastButtonDownState = reading;


	//Up button               
	// read the state of the switch into a local variable:
	reading = digitalRead(buttonPinUp);

	// check to see if you just pressed the Down button 
	// (i.e. the input went from LOW to HIGH),  and you've waited 
	// long enough since the last press to ignore any noise:  

	// If the switch changed, due to noise or pressing:
	if (reading != lastButtonUpState) {
		// reset the debouncing timer
		lastUpDebounceTime = millis();
	}

	if ((millis() - lastUpDebounceTime) > debounceDelay) {
		// whatever the reading is at, it's been there for longer
		// than the debounce delay, so take it as the actual current state:
		buttonUpState = reading;
		lastUpDebounceTime = millis();;
	}

	// save the reading.  Next time through the loop,
	// it'll be the lastButtonState:
	lastButtonUpState = reading;

	//records which button has been pushed
	if (buttonEnterState == HIGH) {
		lastButtonPushed = buttonPinEnter;

	}
	else if (buttonEscState == HIGH) {
		lastButtonPushed = buttonPinEsc;

	}
	else if (buttonDownState == HIGH) {
		lastButtonPushed = buttonPinDown;

	}
	else if (buttonUpState == HIGH) {
		lastButtonPushed = buttonPinUp;

	}
	else {
		lastButtonPushed = 0;
	}
}

void navigateMenus() {
	MenuItem currentMenu = menu.getCurrent();
	switch (lastButtonPushed) {
	case buttonPinEnter:
		if (!(currentMenu.moveRight())) {
			menu.use();
		}
		else {
			menu.moveRight();
		}
		break;
	case buttonPinEsc:
		digitalWrite(ledPlayng, HIGH);
		digitalWrite(ledSetting, LOW);
		menu.toRoot();
		break;
	case buttonPinDown:
		menu.moveDown();
		break;
	case buttonPinUp:
		menu.moveUp();
		break;
	}

	lastButtonPushed = 0; //reset the lastButtonPushed variable
}

/*
*
CALIBRAZIONE
*
*/

void calibration() {
	lcd.clear();
	lcd.setCursor(0, 0);
	lcd.print("    CALIBRATION     ");
	lcd.setCursor(0, 1);
	lcd.print("Please, push ribbon  ");
	lcd.setCursor(0, 2);
	lcd.print("controller with the ");
	lcd.setCursor(0, 3);
	lcd.print("max useful force!   ");

	int startTime;
	int whileTime;

	startTime = millis();
	whileTime = 0;

	//Imposto gli upper e lower bound per il sensore di pressione
	sensorForceMinValue = 1023;
	sensorForceMaxValue = 0;

	while (whileTime < 5000) {
		sensorForceValue = analogRead(sensorForce);

		// record the maximum force sensor value
		if (sensorForceValue > sensorForceMaxValue) {
			sensorForceMaxValue = sensorForceValue;
		}

		// record the minimum force sensor value
		if (sensorForceValue < sensorForceMinValue) {
			sensorForceMinValue = sensorForceValue;
		}

		whileTime = millis() - startTime;
	}

	int sensorForceNewMinValue = sensorForceMinValue / 4;
	int sensorForceNewMaxValue = sensorForceMaxValue / 4;

	EEPROM.write(addrSensorForceMin, sensorForceNewMinValue);
	EEPROM.write(addrSensorForceMax, sensorForceNewMaxValue);

	lcd.clear();
	lcd.setCursor(0, 0);
	lcd.print("    CALIBRATION     ");
	lcd.setCursor(0, 1);
	lcd.print("Calibration ended!  ");

	delay(1000); //delay to read ended calibration message   
}

/*
*
WIRELESS MODE
*
*/

void SetWireless() {
	lcd.clear();
	lcd.setCursor(0, 1);
	lcd.print("    Wi-Fi SETTING   ");
	lcd.setCursor(0, 3);
	lcd.print("   Please, wait...  ");
	delay(2000);

	lcd.clear();
	lcd.setCursor(0, 0);
	lcd.print("   Connecting to:   ");
	lcd.setCursor(0, 1);
	lcd.print(ssid);

	//WiFi.begin(ssid, password);

	int primoLoop = 0;
	int indiceColonna = 7;

	int debug = 0;

	digitalWrite(ledWired, LOW);

	//while (WiFi.status() != WL_CONNECTED) {
	while (debug<20) {
		digitalWrite(ledWifi, HIGH);
		delay(500);
		if (primoLoop == 0) {
			lcd.setCursor(0, 3);
			lcd.print("Waiting");
			primoLoop = 1;
			debug++;
		}
		else {
			if (indiceColonna == 20) {
				lcd.setCursor(0, 3);
				lcd.print("Waiting             ");
				indiceColonna = 7;
				debug++;
			}
			else {
				lcd.setCursor(indiceColonna, 3);
				lcd.print(".");
				indiceColonna++;
				debug++;
			}
		}

		digitalWrite(ledWifi, LOW);
		delay(500);
	}

	digitalWrite(ledWifi, HIGH);

	lcd.clear();
	lcd.setCursor(0, 1);
	lcd.print("  Wi-Fi Connected!  ");
	lcd.setCursor(0, 3);
	lcd.print("IP:");
	lcd.setCursor(4, 3);

	//lcd.print(WiFi.localIP());
	lcd.print("192.168.1.1");

	delay(3000);

	lcd.clear();
	lcd.setCursor(0, 0);
	lcd.print("OK! Now enable a");
	lcd.setCursor(0, 2);
	lcd.print("PC rtpMIDI session");
	delay(3000);

	lcd.clear();
	lcd.setCursor(0, 0);
	lcd.print("Add device Host/Port");
	lcd.setCursor(0, 2);

	//lcd.print(WiFi.localIP());
	lcd.print("192.168.1.1");

	lcd.setCursor(14, 2);
	lcd.print(":5004");
	delay(3000);

	lcd.clear();
	lcd.setCursor(0, 0);
	lcd.print("    Finally    ");
	lcd.setCursor(0, 2);
	lcd.print("Press Connect button");

	//AppleMIDI.begin("test");

	AppleMIDI.OnConnected(OnAppleMidiConnected);
	AppleMIDI.OnDisconnected(OnAppleMidiDisconnected);

	digitalWrite(ledWired, LOW);
	digitalWrite(ledWifi, HIGH);

}

void OnAppleMidiConnected(uint32_t ssrc, char* name) {
	isConnected = true;

	lcd.clear();
	lcd.setCursor(0, 0);
	lcd.print("Connected to session");
	lcd.setCursor(0, 2);
	lcd.print(name);
}

void OnAppleMidiDisconnected(uint32_t ssrc) {
	isConnected = false;
	lcd.clear();
	lcd.setCursor(0, 1);
	lcd.print("DISCONNECTED");
}

/*
*
SETTAGGIO CREDENZIALI WIRELESS
*
*/

void SetWifiCredential(int value) {

	Serial3.begin(9600);
	String message;
	int primoLoop = 0;
	int indiceColonna = 7;

	switch (value) {
	case 0:
		lcd.clear();
		lcd.setCursor(0, 1);
		lcd.print("SSID setup procedure");
		delay(4000);
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("Connect bluetooth");
		lcd.setCursor(0, 1);
		lcd.print("editor and follow ");
		lcd.setCursor(0, 2);
		lcd.print("istructions on lcd");
		delay(4000);
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("Send SSID to:");
		lcd.setCursor(0, 1);
		lcd.print("BT device: HC-06");
		lcd.setCursor(0, 2);
		lcd.print("Pin: 1234");
		digitalWrite(ledWired, LOW);

		while (!Serial3.available()) {
			digitalWrite(ledWifi, HIGH);
			delay(500);
			if (primoLoop == 0) {
				lcd.setCursor(0, 3);
				lcd.print("Waiting");
				primoLoop = 1;
			}
			else {
				if (indiceColonna == 20) {
					lcd.setCursor(0, 3);
					lcd.print("Waiting             ");
					indiceColonna = 7;
				}
				else {
					lcd.setCursor(indiceColonna, 3);
					lcd.print(".");
					indiceColonna++;
				}
			}
			digitalWrite(ledWifi, LOW);
			delay(500);
		}

		digitalWrite(ledWifi, LOW);

		while (Serial3.available()) {
			message += char(Serial3.read());
		}
		id = 0;

		SetParameters(id, message);

		message = "";

		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("SSID setted!");
		lcd.setCursor(0, 1);
		lcd.print("Now connect Wi-Fi");
		lcd.setCursor(0, 2);
		lcd.print("in menu WiFi setting");
		Serial3.println("New SSID setted!");
		delay(300);
		Serial3.println("Now you can disconnect your bluetooth device");
		delay(300);
		Serial3.println("Procedure completed!");
		delay(3000);

		if (currentSendingMode = statusSendWired) {
			digitalWrite(ledWired, HIGH);
			digitalWrite(ledWifi, LOW);
		}
		else {
			digitalWrite(ledWifi, HIGH);
			digitalWrite(ledWired, LOW);
		}

		break;

	case 1:
		lcd.clear();
		lcd.setCursor(0, 1);
		lcd.print("Pwd setup procedure");
		delay(4000);
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("Connect bluetooth");
		lcd.setCursor(0, 1);
		lcd.print("editor and follow ");
		lcd.setCursor(0, 2);
		lcd.print("istructions on lcd");
		delay(4000);
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("Send Password to:");
		lcd.setCursor(0, 1);
		lcd.print("BT device: HC-06");
		lcd.setCursor(0, 2);
		lcd.print("Pin: 1234");
		digitalWrite(ledWired, LOW);
		while (!Serial3.available()) {
			digitalWrite(ledWifi, HIGH);
			delay(500);
			if (primoLoop == 0) {
				lcd.setCursor(0, 3);
				lcd.print("Waiting");
				primoLoop = 1;
			}
			else {
				if (indiceColonna == 20) {
					lcd.setCursor(0, 3);
					lcd.print("Waiting             ");
					indiceColonna = 7;
				}
				else {
					lcd.setCursor(indiceColonna, 3);
					lcd.print(".");
					indiceColonna++;
				}
			}
			digitalWrite(ledWifi, LOW);
			delay(500);
		}

		digitalWrite(ledWifi, LOW);

		while (Serial3.available()) {
			message += char(Serial3.read());
		}
		id = 1;

		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print(message);

		SetParameters(id, message);

		message = "";

		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("Password setted!");
		lcd.setCursor(0, 1);
		lcd.print("Now connect Wi-Fi");
		lcd.setCursor(0, 2);
		lcd.print("in menu WiFi setting");
		Serial3.println("New Password setted!");
		delay(300);
		Serial3.println("Now you can disconnect your bluetooth device");
		delay(300);
		Serial3.println("Procedure completed!");
		delay(3000);

		if (currentSendingMode = statusSendWired) {
			digitalWrite(ledWired, HIGH);
			digitalWrite(ledWifi, LOW);
		}
		else {
			digitalWrite(ledWifi, HIGH);
			digitalWrite(ledWired, LOW);
		}

		break;

	case 2:
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("SSID and Password");
		lcd.setCursor(0, 1);
		lcd.print("setting procedure");
		delay(4000);
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("Connect bluetooth");
		lcd.setCursor(0, 1);
		lcd.print("editor and follow ");
		lcd.setCursor(0, 2);
		lcd.print("istructions on lcd");
		delay(4000);
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("Send SSID to:");
		lcd.setCursor(0, 1);
		lcd.print("BT device: HC-06");
		lcd.setCursor(0, 2);
		lcd.print("Pin: 1234");
		digitalWrite(ledWired, LOW);
		while (!Serial3.available()) {
			digitalWrite(ledWifi, HIGH);
			delay(500);
			if (primoLoop == 0) {
				lcd.setCursor(0, 3);
				lcd.print("Waiting");
				primoLoop = 1;
			}
			else {
				if (indiceColonna == 20) {
					lcd.setCursor(0, 3);
					lcd.print("Waiting             ");
					indiceColonna = 7;
				}
				else {
					lcd.setCursor(indiceColonna, 3);
					lcd.print(".");
					indiceColonna++;
				}
			}
			digitalWrite(ledWifi, LOW);
			delay(500);
		}

		digitalWrite(ledWifi, LOW);

		while (Serial3.available()) {
			message += char(Serial3.read());
		}
		id = 0;

		SetParameters(id, message);

		message = "";


		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("SSID setted!");
		delay(3000);

		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("Send Password to:");
		lcd.setCursor(0, 1);
		lcd.print("BT device: HC-06");
		lcd.setCursor(0, 2);
		lcd.print("Pin: 1234");
		primoLoop = 0;
		indiceColonna = 7;
		while (!Serial3.available()) {
			digitalWrite(ledWifi, HIGH);
			delay(500);
			if (primoLoop == 0) {
				lcd.setCursor(0, 3);
				lcd.print("Waiting");
				primoLoop = 1;
			}
			else {
				if (indiceColonna == 20) {
					lcd.setCursor(0, 3);
					lcd.print("Waiting             ");
					indiceColonna = 7;
				}
				else {
					lcd.setCursor(indiceColonna, 3);
					lcd.print(".");
					indiceColonna++;
				}
			}
			digitalWrite(ledWifi, LOW);
			delay(500);
		}

		digitalWrite(ledWifi, LOW);

		while (Serial3.available()) {
			message += char(Serial3.read());
		}
		id = 1;

		SetParameters(id, message);

		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("Password setted!");
		delay(3000);

		message = "";

		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("All setted!");
		lcd.setCursor(0, 1);
		lcd.print("Now connect Wi-Fi");
		lcd.setCursor(0, 2);
		lcd.print("in menu WiFi setting");
		Serial3.println("New SSID and Password setted!");
		delay(300);
		Serial3.println("Now you can disconnect your bluetooth device");
		delay(300);
		Serial3.println("Procedure completed!");
		delay(3000);

		if (currentSendingMode = statusSendWired) {
			digitalWrite(ledWired, HIGH);
			digitalWrite(ledWifi, LOW);
		}
		else {
			digitalWrite(ledWifi, HIGH);
			digitalWrite(ledWired, LOW);
		}

		break;
	}//switch(value)

	Serial3.end();

} //SetWifiCredential

void SetParameters(int id, String par) {
	if (!Serial3.available())
	{
		if (id == 0) {
			par.toCharArray(ssid, 50);
			lcd.clear();
			lcd.setCursor(0, 0);
			lcd.print("SSID:");
			lcd.setCursor(0, 1);
			lcd.print(ssid);
			Serial3.println("SSID recived");
		}
		else if (id == 1) {
			par.toCharArray(password, 50);
			lcd.clear();
			lcd.setCursor(0, 0);
			lcd.print("Password:");
			lcd.setCursor(0, 1);
			lcd.print(password);
			Serial3.println("Password recived");
		}
		else {
			lcd.clear();
			lcd.setCursor(0, 0);
			lcd.print("Unknown error!");
			lcd.setCursor(0, 1);
			lcd.print("Plese go back and ");
			lcd.setCursor(0, 2);
			lcd.print("repeat the procedure");
			Serial3.println();
			Serial3.println("Unknown Error!");
			delay(300);
			Serial3.println();
			Serial3.println("Plese repeat procedure.");
		}
	}
	delay(3000);
}